﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.IO;
using System.Text;

namespace SASEBO_AES_sample
{
    class Program
    {
        public static int CONT_REG   = 0x0002;  // Control register
        public static int ENCDEC_REG = 0x000C;
        public static int LEN_REG    = 0x0080;
        public static int KEY_REG    = 0x0100;
        public static int IV_REG     = 0x0120;
        public static int IC_REG     = 0x0130;
        public static int DATA_REG   = 0x0140;

        static void Main(string[] args)
        {
            byte num;
            int p = 0;
            int data;

            short enc_dec;
            short mode;    // AES mode
            short kwidth;  // KEY width

            byte[] key  = new byte[17];     // Cipher key
            byte[] ic   = new byte[17];     // Initial Counter
            byte[] iv   = new byte[17];     // Initial Vector
            byte[] din  = new byte[17];     // Plaintext/Ciphertext
            byte[] dout = new byte[17];     // Ciphertext/Plaintext

            //-------- Openning message
            Console.WriteLine("\nSASEBO AES control sample program start");
            SASEBO.ft245rl_interface ctrlif = new SASEBO.FT245RL();
            SASEBO.SASEBO_CMD_interface usb_port = new SASEBO.SASEBO_CMD_interface(ctrlif);

            usb_port.write_short(CONT_REG, 0x0004);   // Reset out
            data = usb_port.read_short(0xfffc);       // FPGA1 Verstion read
            Console.WriteLine("ver  : " + data.ToString("x4") + "\n");

            StreamReader sr = new StreamReader("aes.txt", Encoding.GetEncoding("Shift_JIS"));
            string text = sr.ReadToEnd();
            sr.Close();

            for (int n = 0; n < 10000; n++)
            {
                if (text[p] == '#')
                {
                    p++;
                    switch (text[p])
                    {
                        case 'K':
                            p += 2;
                            for (int i = 0; i < 16; i++)
                            {
                                num = (byte)(toBi(text[p++]) << 4);
                                num = (byte)(num + toBi(text[p++]));
                                key[i] = num;
                            }
                            Console.Write("KEY : ");
                            for (int i = 0; i < 16; i++) Console.Write(toHex(key[i], 2));
                            Console.WriteLine();
                            usb_port.write_b(KEY_REG, key, 16);   // Key register write
                            usb_port.write_short(CONT_REG, 0x0002);   // "Krdy" input
                            break;
                        case 'C':
                            p += 2;
                            for (int i = 0; i < 16; i++)
                            {
                                num = (byte)(toBi(text[p++]) << 4);
                                num = (byte)(num + toBi(text[p++]));
                                ic[i] = num;
                            }
                            Console.Write("IC  : ");
                            for (int i = 0; i < 16; i++) Console.Write(toHex(ic[i], 2));
                            Console.WriteLine();
                            usb_port.write_b(0x0100, ic, 16);         // IC register write
                            usb_port.write_short(CONT_REG, 0x0020);   // "Crdy" input
                            break;
                        case 'I':
                            p += 2;
                            for (int i = 0; i < 16; i++)
                            {
                                num = (byte)(toBi(text[p++]) << 4);
                                num = (byte)(num + toBi(text[p++]));
                                iv[i] = num;
                            }
                            Console.Write("IV  : ");
                            for (int i = 0; i < 16; i++) Console.Write(toHex(iv[i], 2));
                            Console.WriteLine();
                            usb_port.write_b(IV_REG, iv, 16);         // IV register write
                            usb_port.write_short(CONT_REG, 0x0010);   // "Irdy" input
                            break;
                        case 'D':
                            p += 2;
                            for (int i = 0; i < 16; i++)
                            {
                                num = (byte)(toBi(text[p++]) << 4);
                                num = (byte)(num + toBi(text[p++]));
                                din[i] = num;
                            }
                            Console.Write("DATA: ");
                            for (int i = 0; i < 16; i++) Console.Write(toHex(din[i], 2));
                            Console.WriteLine();
                            usb_port.write_b(DATA_REG, din, 16);      // Data register write
                            usb_port.write_short(CONT_REG, 0x0001);   // "Drdy" input
                            break;
                        case 'E':
                            p += 2;
                            enc_dec = (short)(toBi(text[p++]));
                            if ( enc_dec == 0 ) Console.Write("Encrypt\n");
                            else Console.Write("Decrypt\n");
                            usb_port.write_short(ENCDEC_REG, enc_dec);
                            break;
                        case 'M':
                            p += 2;
                            mode = (short)(toBi(text[p++]));
                            Console.Write("MODE: " + mode +"\n");
                            usb_port.write_short(0x0A, mode);
                            break;
                        case 'W':
                            p += 2;
                            kwidth = (short)(toBi(text[p++]));
                            Console.Write("KEY WIDTH: " + kwidth +"\n");
                            usb_port.write_short(0x0E, kwidth);
                            break;
                        default: p++; break;
                    }
                }
                else if (text[p] == '$')
                {
                    p++;
                    if (text[p] == 'R')
                    {
                        Console.Write("AES Reset!!\n");
                        usb_port.write_short(CONT_REG, 0x0004);   // "Reset" input
                    }
                    else if (text[p] == 'O')
                    {
                        usb_port.read_b(0x0180, dout, 16);       // Output register read
                        Console.Write("OUT : ");
                        for (int i = 0; i < 16; i++) Console.Write(toHex(dout[i], 2));
                        Console.WriteLine();
                        Console.WriteLine();
                    }
                    else if (text[p] == 'F') break;
                    p++;
                }
                else p++;
            }

            Console.WriteLine("Press enter.");
            Console.ReadLine();
            ctrlif.close();
        }


        // ----- Text to Binary -------------------
        private static byte toBi(char text)
        {
            byte d;
            switch (text)
            {
                case '0': d = 0x00; break;
                case '1': d = 0x01; break;
                case '2': d = 0x02; break;
                case '3': d = 0x03; break;
                case '4': d = 0x04; break;
                case '5': d = 0x05; break;
                case '6': d = 0x06; break;
                case '7': d = 0x07; break;
                case '8': d = 0x08; break;
                case '9': d = 0x09; break;
                case 'A': d = 0x0A; break;
                case 'B': d = 0x0B; break;
                case 'C': d = 0x0C; break;
                case 'D': d = 0x0D; break;
                case 'E': d = 0x0E; break;
                case 'F': d = 0x0F; break;
                case 'a': d = 0x0A; break;
                case 'b': d = 0x0B; break;
                case 'c': d = 0x0C; break;
                case 'd': d = 0x0D; break;
                case 'e': d = 0x0E; break;
                case 'f': d = 0x0F; break;
                default: d = 0x00; break;
            }
            return d;
        }


        // ----- Binary to Hex -------------------
        private static string toHex(int num, int width)
        {
            string text = "";
            for (int i = 0; i < width; i++)
                text += ((num >> (4 * (width - i - 1))) & 0xF).ToString("X");
            return text;
        }
    }
}

