﻿/*-------------------------------------------------------------------------
 SASEBO / -R / -B / -G controller tester
 
 File name   : SASEBO_ControlInterface.cs
 Version     : Version 1.0
 Created     : Aug/20/2008
 Last update : Aug/20/2008
 Desgined by : Toshihiro Katashita
 
 
 Copyright (C) 2008 AIST
 
 By using this code, you agree to the following terms and conditions.
 
 This code is copyrighted by AIST ("us").
 
 Permission is hereby granted to copy, reproduce, redistribute or
 otherwise use this code as long as: there is no monetary profit gained
 specifically from the use or reproduction of this code, it is not sold,
 rented, traded or otherwise marketed, and this copyright notice is
 included prominently in any copy made.
 
 We shall not be liable for any damages, including without limitation
 direct, indirect, incidental, special or consequential damages arising
 from the use of this code.
 
 When you publish any results arising from the use of this code, we will
 appreciate it if you can cite our webpage
 (http://www.rcis.aist.go.jp/special/SASEBO/).
 -------------------------------------------------------------------------*/ 

using System;
using System.Collections.Generic;
using System.Text;
using System.IO.Ports;

using FTD2XX_NET;


namespace SASEBO
{
  //================================================================ ControlInterface
  interface ControlInterface
  {
    void open();
    void close();
    void write (byte[] dat, int len);
    void read  (byte[] dat, int len);
  }

/*
  //================================================================ RS232
  class RS232 : ControlInterface
  {
    private string port;
    private SerialPort serialport;

    //------------------------------------------------ Constructor
    public RS232 (string port) {
      this.port = port;
    }

    //------------------------------------------------ open()
    public void open() {
      serialport = new SerialPort(port, 19200, Parity.None, 8, StopBits.One);
      serialport.Open();
      serialport.DtrEnable = true;
      serialport.RtsEnable = true;
    }

    //------------------------------------------------ close()
    public void close() {
      serialport.Close();
    }

    //------------------------------------------------ write()
    public void write(byte[] dat, int len) {
      serialport.Write(dat, 0, len);
    }

    //------------------------------------------------ read()
    public void read(byte[] dat, int len) {
      while (serialport.BytesToRead < len) ;
      serialport.Read(dat, 0, len);
    }
  }
 */   

  //================================================================ FT245RL
  class FT245RL : ControlInterface
  {
    FTDI ft245rlport = new FTDI();

    //------------------------------------------------ Constructor
    public FT245RL () {}

    //------------------------------------------------ open()
    public void open() {
      uint num_device = 0;

      ft245rlport.GetNumberOfDevices(ref num_device);
//      System.Diagnostics.Debug.WriteLine("Number of FTDI devices : "+num_device);
      Console.WriteLine("Number of FTDI devices : " + num_device);
      FTDI.FT_DEVICE_INFO_NODE[] device_list = new FTDI.FT_DEVICE_INFO_NODE[num_device];
      ft245rlport.GetDeviceList(device_list);

      foreach (FTDI.FT_DEVICE_INFO_NODE device in device_list) {
/*        System.Diagnostics.Debug.WriteLine("--");
        System.Diagnostics.Debug.WriteLine("Flags  : " + device.Flags);
        System.Diagnostics.Debug.WriteLine("Type   : " + device.Type);
        System.Diagnostics.Debug.WriteLine("ID     : " + device.ID);
        System.Diagnostics.Debug.WriteLine("LocID  : " + device.LocId);
        System.Diagnostics.Debug.WriteLine("Serial : " + device.SerialNumber);
        System.Diagnostics.Debug.WriteLine("Info   : " + device.Description);*/

        Console.WriteLine("--");
        Console.WriteLine("Flags  : " + device.Flags);
        Console.WriteLine("Type   : " + device.Type);
        Console.WriteLine("ID     : " + device.ID);
        Console.WriteLine("LocID  : " + device.LocId);
        Console.WriteLine("Serial : " + device.SerialNumber);
        Console.WriteLine("Info   : " + device.Description);
      }

      ft245rlport.OpenByIndex(1);
//        if (!ft245rlport.IsOpen) System.Diagnostics.Debug.WriteLine("Failed to open device.");
        if (!ft245rlport.IsOpen) Console.WriteLine("Failed to open device.");
        else Console.WriteLine("Successfull to open device.");
      ft245rlport.ResetDevice();
   }

    //------------------------------------------------ close()
    public void close() {
      ft245rlport.Close();
    }

    //------------------------------------------------ write()
    public void write(byte[] dat, int len) {
      uint wlen = 0;
        ft245rlport.Write(dat, len, ref wlen);
    }
    
    //------------------------------------------------ read()
    public void read(byte[] dat, int len) {
      uint rlen = 0;
      ft245rlport.Read(dat, (uint)len, ref rlen);
    }
  }
}

