﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.IO;
using System.Text;

namespace SASEBO_SHA256_sample
{
    class Program
    {
        public static int CONT_REG = 0x0002;  // Control register address
        public static int DIN_REG  = 0x0140;
        public static int DOUT_REG = 0x0180;
        public static int CONT_RST = 0x0004;  // Reset bit

        static void Main(string[] args)
        {
            byte num;
            int p = 0;
            int data;


            byte[] key = new byte[17];     // Cipher key
            byte[] din = new byte[17];     // Plaintext/Ciphertext
            byte[] dout = new byte[33];    // Ciphertext/Plaintext
            byte[] tmp = new byte[10000];  //

            //-------- Openning message
            Console.WriteLine("\nSASEBO CMAC control sample program start");
            SASEBO.ft245rl_interface ctrlif = new SASEBO.FT245RL();
            SASEBO.SASEBO_CMD_interface usb_port = new SASEBO.SASEBO_CMD_interface(ctrlif);

            usb_port.write_short(CONT_REG, 0x0004);   // Reset out
            data = usb_port.read_short(0xfffc);       // FPGA1 Verstion read
            Console.WriteLine("ver  : " + data.ToString("x4") + "\n");

            StreamReader sr = new StreamReader("sha256.txt", Encoding.GetEncoding("Shift_JIS"));
            string text = sr.ReadToEnd();
            sr.Close();
            Console.WriteLine("SASEBO SHA-256 control sample program start");


            for (int n = 0; n < 10000; n++)
            {
                if (text[p] == '#')
                {
                    p++;
                    switch (text[p])
                    {
                       case 'D':
                            p += 2;
                            for (int i = 0; i < 4; i++)
                            {
                                num = (byte)(toBi(text[p++]) << 4);
                                num = (byte)(num + toBi(text[p++]));
                                din[i] = num;
                            }
                            Console.Write("DATA: ");
                            for (int i = 0; i < 4; i++) Console.Write(toHex(din[i], 2));
                            Console.WriteLine();
                            usb_port.write_b(DIN_REG, din, 4);        // Data register write
                            usb_port.write_short(CONT_REG, 0x0001);   // "Drdy" input
                            break;
                        default: p++; break;
                    }
                }
                else if (text[p] == '$')
                {
                    p++;
                    if (text[p] == 'R')
                    {
                        Console.Write("SHA256 Reset!!\n");
                        usb_port.write_short(CONT_REG, 0x0004);   // "Reset" input
                    }

                    else if (text[p] == 'O')
                    {
                        usb_port.read_b(DOUT_REG, dout, 32);         // Output register read
                        Console.Write("OUT : ");
                        for (int i = 0; i < 32; i++) Console.Write(toHex(dout[i], 2));
                        Console.WriteLine();
                        Console.WriteLine();
                    }
                    else if (text[p] == 'F') break;
                    p++;
                }
                else p++;
            }

            Console.WriteLine("Press enter.");
            Console.ReadLine();
            ctrlif.close();
        }

        
        // ----- Text to Binary -------------------
        private static byte toBi(char text)
        {
            byte d;
            switch (text)
            {
                case '0': d = 0x00; break;
                case '1': d = 0x01; break;
                case '2': d = 0x02; break;
                case '3': d = 0x03; break;
                case '4': d = 0x04; break;
                case '5': d = 0x05; break;
                case '6': d = 0x06; break;
                case '7': d = 0x07; break;
                case '8': d = 0x08; break;
                case '9': d = 0x09; break;
                case 'A': d = 0x0A; break;
                case 'B': d = 0x0B; break;
                case 'C': d = 0x0C; break;
                case 'D': d = 0x0D; break;
                case 'E': d = 0x0E; break;
                case 'F': d = 0x0F; break;
                case 'a': d = 0x0A; break;
                case 'b': d = 0x0B; break;
                case 'c': d = 0x0C; break;
                case 'd': d = 0x0D; break;
                case 'e': d = 0x0E; break;
                case 'f': d = 0x0F; break;
                default: d = 0x00; break;
            }
            return d;
        }


        // ----- Binary to Hex -------------------
        private static string toHex(int num, int width)
        {
            string text = "";
            for (int i = 0; i < width; i++)
                text += ((num >> (4 * (width - i - 1))) & 0xF).ToString("X");
            return text;
        }
    }
}
